/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alipay.sofa.lookout.gateway.core.prototype.lifecycle;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author xiangfeng.xzc
 * @date 2018/11/14
 */
public abstract class LifeCycleSupport implements LifeCycle {
    private final Logger  LOGGER = LoggerFactory.getLogger(getClass());

    private AtomicInteger state  = new AtomicInteger(0);

    @Override
    public void start() {
        if (state.compareAndSet(0, 1)) {
            doStart();
        } else {
            LOGGER.error("start状态错误");
        }
    }

    /**
     * 空实现, 子类如果有需要的话可以重写
     */
    protected void doStart() {
    }

    @Override
    public void stop() {
        if (state.compareAndSet(1, 2)) {
            doStop();
        } else {
            LOGGER.error("stop状态错误");
        }
    }

    /**
     * 空实现, 子类如果有需要的话可以重写
     */
    protected void doStop() {
    }
}
